import $ from "./jquery.module.js";
import SetConfig from "./set_config_class.js";
import * as dialog from "./dialog.js";
import {ItemPlacement} from "./item_placement_class.js";
import * as collisions from "./detect_collisions.js";
import * as audio from "./audio.js";

/* This section passes the information in the config file to the js file */
let config;
let catchItems =  [];
let catcher;
let $catcher;
let itemDisplay;
let dropSlots = [];
let catcherLimit = 1000;

let setConfigHandler = new SetConfig();

let lastTime = 0;
let dropItemCount = 0;
let slotIdx = 0; // Remember slot between spawnings so we don't repeat the same one twice
let catchItemInfo = {};
let decoyItemsCaught = 0;
let gameStopped = false;

let dropItemInterval = null;
let collisionTextInterval = null;

let leftButtonPressed = false;
let rightButtonPressed = false;

// Function imported from set_config_class.js that resolves the config object, so we can get data from config file ;
function setConfig(config) {
    setConfigHandler.setConfig(config);
}

/* End config section */
 // ASSIGNS IMAGES TO THE BACKGROUND, CATCHER THING AND THE ITEMS TO CATCH ;
function initializeCatchModule() {
    $("#background-img").attr("src",config.options.pageBackground) ;

    for (const item of config.displayItems) {
        switch (item.catchType) {
            case "catcherItem":
                catcher = item;
                break;
            case "catchItem":
            case "decoyItem":
                catchItems.push(item);
                catchItemInfo[item.displayItemID] = {item, count: 0};
                break;
            default:

                break;
        }
    }

    dropSlots = config.dropSlots;

    let itemPlacementConfig = {...config.itemPlacementConfig};
    catcher.cssProperties = {"z-index": 10};
    itemPlacementConfig.displayItems = [catcher]; // Start with just the catcher item
    itemDisplay = new ItemPlacement(itemPlacementConfig);
}

function clamp(x, a, b) {
    return Math.max( a, Math.min(x, b) );
}

function moveCatcher(direction, ts) {
    const catcherSpeed = 5000;
    const delta = 20*(direction === "left" ? -1 : 1);
    let newX = clamp(parseFloat($catcher.css("left")) + delta, 0, catcherLimit);
    $catcher.css("left", newX);
}

function initializeCounter() {
    function makeCounter({label, id, numberRequired}) {
        const $counterForThing = $(`<div class="counter"></div>`);
        $counterForThing.append($(`<div id="${id}-label" class="counter-label">${label}</div>`));
        $counterForThing.append($(`<div"><span  id="${id}-count" class="counter-count">0</span>/${numberRequired}</div>`))
        return $counterForThing;
    }

    function counterText(label, id, numberRequired) {
        return {label, id, numberRequired};
    }


    const $counterDiv = $(`<div id="counter-div"></div>`);
    $("#counter").append($counterDiv);
    const count_items = config.displayItems.filter(item => item.catchType === "catchItem") ;
    for (let i = 0; i < count_items.length  ; i++) {
        $counterDiv.append(makeCounter(counterText(count_items[i].displayItemID,   count_items[i].displayItemID, count_items[i].numberRequired)));
    }
}

function keydownHandler(evt) {
    switch(evt.key) {
        case "ArrowLeft":
            moveCatcher("left", Date.now());
            break;

        case "ArrowRight":
            moveCatcher("right", Date.now());
            break;

        default:
            break;
    }
    lastTime = evt.timestamp;
}

function keyupHandler(evt) {
    switch(evt.key) {
        case "ArrowLeft":
            lastTime = 0;
            break;

        case "ArrowRight":
            lastTime = 0;
            break;

        default:
            break;
    }
}

function arrowButtonHandler(evt) {
    const tgt = evt.delegateTarget;
    switch($(tgt).attr("id")) {
        case "left-btn":
            switch(evt.type) {
                case "mousedown":
                    leftButtonPressed = true;
                    break;

                case "mouseup":
                case "mouseleave":
                    leftButtonPressed = false;
                    break;
            }

            break;
        case "right-btn":
            switch(evt.type) {
                case "mousedown":
                    rightButtonPressed = true;
                    break;

                case "mouseup":
                case "mouseleave":
                    rightButtonPressed = false;
                    break;
            }

            break;
        default:
            break;
    }
}
function dropItem(item, loc) {
    let itemCopy = {... item};
    dropItemCount++;
    const itemID = item.displayItemID + "_" + dropItemCount.toString();
    itemCopy.location = loc;
    itemCopy.displayItemID = itemID;
    if ("dropScaling" in itemCopy) {
        itemCopy.scaling = itemCopy.dropScaling;
    }
    itemDisplay.addDisplayItem(itemCopy);
    $(`#${itemID}`)
        .addClass(["drop-animation", "drop-item"])
        .attr("data-displayitemid", item.displayItemID)
        .on("animationend", handleAnimationEnd);
}

function scheduleItemDrops() {
    const numOfDropSlots = dropSlots.length;
    const numOfCatchItems = catchItems.length;
    dropItemInterval = setInterval(() => {
        const catchIdx = getRandomInt(0, numOfCatchItems);
        slotIdx = (slotIdx + getRandomInt(1, numOfDropSlots)) % numOfDropSlots; // Guarantees the new slot number is different
        dropItem(catchItems[catchIdx], dropSlots[slotIdx]);
    }, 1000)
}

function scheduleCollisionTest() {
    collisionTextInterval = setInterval(collisionTest, 50);
}

function handleAnimationEnd(evt) {
    $(evt.currentTarget).remove();
}

function getRandomInt(min, max) {
    min = Math.ceil(min);
    max = Math.floor(max);
    return Math.floor(Math.random() * (max - min) + min); // The maximum is exclusive and the minimum is inclusive
}

function collisionTest() {
    if (leftButtonPressed) {
        moveCatcher("left");
    }
    if (rightButtonPressed) {
        moveCatcher("right");
    }
    const {foundCollision, collider} = collisions.testCollisionForAllJQueryElements($catcher, $(".drop-item"));
    if (foundCollision) {
        processCaughtItem(collider);
        collider.remove();
    }
}

function processCaughtItem(collider) {
    const id = collider.attr("data-displayitemid");
    const catchInfo = catchItemInfo[id];
    const numberRequired = catchInfo.item.numberRequired;
    catchInfo.count++;
    if (catchInfo.item.catchType === "catchItem") {
        const countDisplayID = `#${id}-count`;
        if (catchInfo.count <= numberRequired) {
            $(countDisplayID).html(catchInfo.count);
        }
        audio.playAudioFile(config.options.correctSoundPath);
    }
    else if (catchInfo.item.catchType === "decoyItem") {
        decoyItemsCaught++;
    }
    checkForFinish();
}

function checkForFinish() {
    let finished = true;
    for (const [key, catchInfo] of Object.entries(catchItemInfo)) {
        if (catchInfo.item.catchType === "catchItem") {
            if (catchInfo.count < catchInfo.item.numberRequired) {
                finished = false;
                break;
            }
        }
    }
    if (finished) {
        stopGame();
        dialog.showDialogAsync(config.options.successText)
            .then(() => {window.location.href = config.options.completionURL;});
        $("#layer1").css("background-image","url('" + config.options.winGameImage + "')") ;
        $(".dialog7gen").css("border-radius","20px") ;


    }
}

function showHint() {
    dialog.showDialogAsync(config.options.hintText + getItemsLeftList());
}

function getItemsLeftList() {
    let text = "";
    for (const [key, catchInfo] of Object.entries(catchItemInfo)) {
        if (catchInfo.item.catchType === "catchItem") {
            const itemsLeft = catchInfo.item.numberRequired - catchInfo.count;
            if (itemsLeft > 0) {
                const itemName = catchInfo.item.itemName;
                text += `<p>${itemName} : ${itemsLeft}</p>`;
            }
        }
    }
    return text;
}

function getIntroText() {
    return config.options.instructions + getItemsLeftList();
}

function stopGame() {
    clearInterval(dropItemInterval);
    clearInterval(collisionTextInterval);
    $catcher.remove();
    $(".drop-item").remove();

}

$(async () => {
    config = await setConfigHandler.getConfigAsync();

    initializeCatchModule();
    itemDisplay.onDocumentReady();
    $catcher = $(`#${catcher.displayItemID}`);
    catcherLimit = parseFloat($(`#${config.itemPlacementConfig.backgroundDivID}`).css("width"))
        - parseFloat($catcher.css("width"));
    initializeCounter();

    $(document).on("keydown", keydownHandler);
    $(document).on("keyup", keyupHandler);
    $("#hint").on("click", showHint);

    $("#left-btn")
        .on("mousedown",   arrowButtonHandler)
        .on("mouseup",     arrowButtonHandler)
        .on("mouseleave",  arrowButtonHandler);

    $("#right-btn")
        .on("mousedown",  arrowButtonHandler)
        .on("mouseup",    arrowButtonHandler)
        .on("mouseleave", arrowButtonHandler);

    await dialog.showDialogAsync(getIntroText());
    // Anything here executes after the dialog closes ;
    $("#hint").removeClass("d-none") ;
    scheduleItemDrops();
    scheduleCollisionTest();
});

export {setConfig};