// collect_objects_game.js

import $ from "./jquery.module.js";
import SetConfig from "./set_config_class.js";
import * as audio from "./audio.js";
import {setAudioFile} from "./audio.js";
let setConfig = new SetConfig();

$(() => {
    setConfig.then((config) => {

        const gridParameters = config.gridParameters;

        const characterGridSlots = gridParameters.characterGridSlots;
        const characterImage = config.options.characterImage; // Image for the character that is collecting things.  ;
        const gameBoardSelector = "#game-board";


        const characterLimit =  {
            lowerLimit: 0,   // Recalculated at beginning of game
            upperLimit: 580, // Recalculated at beginning of game
            lowerDelta: gridParameters.characterLimit.lowerDelta,
            upperDelta: gridParameters.characterLimit.upperDelta,
        } ; // Object giving limits to character movement on the game board ;
        // Make field item grid 1000px by 600px, and the objects will all be 50px by 50px
// So the grid is 20 by 12.
// But, let's leave a space between grid items horizontal and vertical,
// so the grid will only be 10 by 6.

        const originalGridWidth =  gridParameters.originalGridWidth;
        const gridOriginalOrigin =    {
            left: gridParameters.gridOriginalOrigin.left,
            top: gridParameters.gridOriginalOrigin.top
        } ;

        const originalGridPitch = gridParameters.originalGridPitch;
        let gridPitch = originalGridPitch; // Will be changed when page loaded. Used to scale and position objects;

        const gridWidth = gridParameters.gridWidth;
        const gridHeight = gridParameters.gridHeight;

        const grid =  config.grid;

        const scoreLabel = config.options.scoreLabel;
        const healthIcon = config.options.healthIcon;
        const instructions = config.options.instructionsPage1
        $("#instructions_help").html(instructions + "<img src=\"../../blocks/game_theme/theme/icons/play.png\" alt=\"play\" class='' id=\"play\" >");
        let characterY;
        let $character;
        let done = false;
        let rewardsHarvested = 0; // Number of rewards the player has collected ;
        let maxRewards = 0;  // Reset below. This is the number of rewards a player must get to win the game ;
        let healthPoints = config.options.maxHealthPoints;

        let screenImageWidth;
        let screenImageHeight;

        const displayItems = config.displayItems;
        let displayItemMap = []; // Array that holds image src and type for each symbol 0 (reward) and 1 and 2 (obstacles) ;
        let displayItemDiv = {}; // Will be used to hold a div with image and class for rewards and obstacles on the game board ;

        let bkgImg = `<img src="${config.options.backgroundImage}" alt=\"Background\">`;
        $("#game-board").append(bkgImg);



        addHealthIcons();  // Function to add icons for health points at top of screen. One icon for each 'life' ;

        /*
            CREATES AN ITEM MAP, WITH EACH DISPLAY ITEM FROM THE CONFIG FILE  ASSOCIATED WITH A SYMBOL
            THAT MATCHES THE GRID, A SRC FILE FOR THAT  ITEM AND WHAT TYPE IT IS, REWARD OR OBSTACLE
         */
        displayItems.forEach((displayItem) => {
            let symbol = displayItem.itemSymbol;
            let src = displayItem.itemImage;
            let type = displayItem.itemType;
            displayItemMap[symbol] = displayItem;
            displayItemDiv[symbol] = `<div class="field-item ${type}"><img src="${src}" draggable="false"></div>`;
        });

// Calculate total number of rewards
        // Don't really need this because rewards are fixed in grid, but useful if we  decide to change grid ;
        for (let row of grid) {
            for (let item of row) {
                if (item in displayItemMap && displayItemMap[item].itemType === "reward") {
                    ++maxRewards;
                }
            }
        }

        function getRandomInt(max) {
            return Math.floor(Math.random() * max);
        }

        /*
        Appends a character to the gameboard with a class of 'character' and an image source as
        specified in the config file (which was given to the variable characterImage above)
        The character top CSS value is randomly selected from one of the slots specified in the
        characterGridSlots. Since this new div has a class of character, the animation from
        collect_objects.css is applied to move it across the screen.
        When the character gets to the end of the screen it is removed and another character created
         */
        function addCharacter(slot) {
           $character = $(`<div class="character" draggable="false"></div>`)
           $character.appendTo($(gameBoardSelector));
           $character.html(`<img src="${characterImage}" >`);
            let topPosition = `${characterSlots[slot]} px`;
            characterY = characterSlots[slot];
           $character.css("top", characterSlots[slot]);
           $character.on("animationend", () => {
               $character.remove();
                spawnOneCharacter();
            });
        }

       // RANDOMLY PICKS A VALUE (nextSlot) FROM THE characterSlots ARRAY AND ADDS
       // A CHARACTER TO THE SCREEN AT THE POSITION SPECIFIED WITH THE nextSlot ;
        function spawnOneCharacter() {
            const nextSlot = getRandomInt(characterSlots.length);
            addCharacter(nextSlot);
        }

        /* KEY HANDLER FUNCTIONS */

        // MOVES CHARACTER UP BY CHANGING TOP CSS VALUE ;
        function moveCharacterUp(evt) {
            characterY -= characterInc;
            characterY = (characterY < characterLimit.lowerLimit) ? characterLimit.lowerLimit : characterY;
           $character?.css("top", characterY.toString() + "px");
        }

        // MOVES CHARACTER DOWN BY CHANGING TOP CSS VALUE ;
        function moveCharacterDown(evt) {
            characterY += characterInc;
            characterY = (characterY > characterLimit.upperLimit) ? characterLimit.upperLimit : characterY;
           $character?.css("top", characterY.toString() + "px");
        }

        // If up arrow is pressed moves the character up ;
        // If down arrow is pressed moves the charadcter down ;
        function handleKeyboardEvent(event) {
            let key = event.key;
            switch (key) {
                case "ArrowUp":
                    moveCharacterUp();
                    break;
                case "ArrowDown":
                    moveCharacterDown();
                    break;
                default:
                    break;
            }
        }
       /* End of key handler functions.  */

/* FILLS THE GRID WITH OBJECTS, POSITIONED USING gridPitch VALUE */

        function itemClass(y, x) {
            return `coord-${y}-${x}`;
        }

        /* ADD DIV TO GRID WITH IMAGE CORRESPONDING TO NUMBER IN CONFIG FILE
          AND POSITION OBJECTS AT PX BASED ON gridPitch AND GRID ORIGIN PROPERTIES
         */
        function fillGrid() {
            let x, y, object, posX, posY;
            let $object;

            for (y = 0; y < gridHeight; y++) {
                for (x = 0; x < gridWidth; x++) {
                    object = grid[y][x];
                    if (object !== " ") {
                        posX = gridOrigin.left + (2 * x + 1) * gridPitch;
                        posY = gridOrigin.top + (2 * y + 1) * gridPitch;

                        $object = $(displayItemDiv[object]);

                        $object.appendTo($(gameBoardSelector));
                        $object.css({
                            top: `${posY}px`,
                            left: `${posX}px`
                        });
                        $object.addClass(itemClass(y, x)); // So we can find it later

                    }
                }
            }
        }


        // Returns left and top positions as a fraction of gridPitch ;
        function mapPosToGrid(pos) {
            return {
                x: Math.floor(((pos.x - gridOrigin.left - 1) / 2) / gridPitch),
                y: Math.floor(((pos.y - gridOrigin.top - 1) / 2) / gridPitch),
            }
        }


/*
    THESE TWO FUNCTIONS HANDLE THE HEALTH ICONS AND HEALTH POINTS
    THE FIRST PUTS UP AN ICON FOR EVERY HEALTH POINT THE PLAYER HAS. T
    THE SECOND FUNCTION DECREMENTS THE HEALTH POINTS AND SETS A VALUE (DONE) TO TRUE
    WHEN THE PLAYER HAS NO MORE HEALTH POINTS
 */
        function addHealthIcons() {
            let i;
            let $healthDiv = $("#health");
            for (i = 1; i <= healthPoints; i++) {
                $healthDiv.append(`<img id="t${i}" class="health-icon" src="${healthIcon}" alt="Health Icon">`)
            }
        }

        function loseHealthPoint() {
            $(`#t${healthPoints}`).addClass("d-none");
            --healthPoints;
            if (healthPoints === 0) {
                done = true;
            }
        }
/* END HEALTH POINTS SECTION  */

        function winGame() {
            $("#game-board").remove();
            $("#movement-buttons").remove();
            $("#header").remove();
            let nextArrow = `<img src="../../game_theme/theme/icons/next.png" class="nextArrow float-end" >` ;
            if (config.options.winGameImage) {
                $("#layer1").css("background-image", "url(" + config.options.winGameImage + ")" ) ;
            }
            $("#win-game").removeClass("d-none").append(config.options.successText).append(nextArrow);
            $(".nextArrow").on("click", () => {window.location.href = config.options.completionURL;});

        }

        function loseGame() {
            $("#game-board").remove();
            $("#movement-buttons").remove();
            $("#header").remove();
            $("#lose-game").removeClass("d-none").append(config.options.loseText);
        }

        function testForCollision() {
            let collision = false;
            let corner, x, y;
            let obj = " ";

            const yPos = parseFloat($character.css("top"));
            const xPos = parseFloat($character.css("left"));

            let x1 = xPos - gridOrigin.left;
            let y1 = yPos - gridOrigin.top;


            // Get the grid position of each corner.
            const corner1 = mapPosToGrid(
                {
                    x: xPos,
                    y: yPos
                }
            );

            const corner2 = mapPosToGrid(
                {
                    x: xPos + characterWidth,
                    y: yPos
                }
            );

            const corner3 = mapPosToGrid(
                {
                    x: xPos,
                    y: yPos + characterHeight
                }
            );

            const corner4 = mapPosToGrid(
                {
                    x: xPos + characterWidth,
                    y: yPos + characterHeight
                }
            );

                for (corner of [corner1/*, corner2, corner3, corner4*/]) {
                let [x, y] = [corner.x, corner.y];
                if (y < 0) {
                    return;
                }
                x = x < 0 ? 0 : x;

                if (y < grid.length && x < grid[y].length) {
                    obj = grid[y][x];
                    if (obj !== " ") {
                        collision = true;
                        grid[y][x] = " ";
                        $(`.${itemClass(y, x)}`).remove();

                        break;
                    }
                }
            }




            if (collision) {
                switch (obj) {
                    case "0":
                        setAudioFile(config.options.goodSound) ;
                        audio.play(true) ;
                        rewardsHarvested++;
                        $("#score").text(`${scoreLabel} ${rewardsHarvested}`)
                        done = rewardsHarvested >= maxRewards;
                        break;

                    case "1":
                    case "2":
                        setAudioFile(config.options.badSound) ;
                        audio.play(true) ;
                        loseHealthPoint();
                        break;

                    case " ":
                    default:
                        break;

                }
            }
        }

        function keepTesting() {
            if (done) {
                if (healthPoints <= 0) {
                    loseGame();
                } else {
                    winGame();
                }
            } else {
                testForCollision();
                setTimeout(keepTesting, 100);
            }
        }

        function startGame() {
            $(document).on("keydown", handleKeyboardEvent);
            $("#instructions1").addClass("d-none");
            $("#game-container").removeClass('d-none') ;
            $("#score").removeClass("d-none");
            $("#health").removeClass("d-none");
            fillGrid();
            setTimeout(() => {
                spawnOneCharacter();
                keepTesting();
            }, 5000);
        }

        function reloadGame() {
            window.location.reload();
        }

        $("#score").text(`${scoreLabel} 0`)

        screenImageWidth = parseFloat($("#game-board img").css("width"));
        screenImageHeight = parseFloat($("#game-board img").css("height"));
        let gridScale = screenImageWidth / originalGridWidth
        gridPitch = originalGridPitch * gridScale;
        const characterHeight = gridPitch;
        const characterWidth = gridPitch;
        const characterInc = gridPitch;
        const gridOrigin = {
            left: gridOriginalOrigin.left * gridScale,
            top: gridOriginalOrigin.top * gridScale
        };

        // console.log(`gridOrigin = ${gridOrigin.left}, ${gridOrigin.top}`);
        const characterSlots = characterGridSlots.map((slot) => gridOrigin.top + slot * gridPitch);
        characterLimit.lowerLimit = gridOrigin.top + 1 + characterLimit.lowerDelta * gridScale;
        characterLimit.upperLimit = gridOrigin.top + 2 * gridHeight * gridPitch + characterLimit.upperDelta * gridScale;
        $("#play").on("click", startGame);
        $("#tryagain").on("click", reloadGame);
        $("#move-up").on("click", moveCharacterUp);
        $("#move-down").on("click", moveCharacterDown);

    });
});

export default setConfig;