// home.js module

import * as config from "./home_config.js";
import {hailMaryRoots} from "./home_config.js";

let homeDirectoryCache = null;

function initializeHomeDirectory() {
    // Determine home directory and store in session storage
    // NOTE: this depends being at the top of the site hierarchy
    // when this is called.
    // Normally, this is called from the login page
    let slashChar = "/";
    let currentPath = window.location.pathname;
    let slashIndex = currentPath.lastIndexOf(slashChar);
    if (slashIndex < 0) { // Couldn't find a "/"
        // Maybe some Windows weirdness is going on
        slashChar = "\\"; // Really means one backslash
        slashIndex = currentPath.lastIndexOf(slashChar);
        if (slashIndex < 0) {
            return;
        }
    }
    homeDirectoryCache = currentPath.slice(0, slashIndex) + slashChar;
    sessionStorage.setItem(config.homeDirectoryKey, homeDirectoryCache);
}

function nthIndex(str, pat, n){
    let L= str.length, i= -1;
    while(n-- && i++<L){
        i= str.indexOf(pat, i);
        if (i < 0) break;
    }
    return i;
}

function homeDirectoryHailMaryInit() {
    // This is called if homeDirectory is requested before the proper initialization has been called
    // In this case we are probably testing in a browser and we might be in a subdirectory; in this case
    // we are going to:
    // 1) Try to see if hailMaryRoot from home_config.js is in the current path; if so we are going to
    //    assume that the first occurence in the path is the root, so the home directory will be
    //    of the form /.../${config.hailMaryRoot}/
    // 2) If even that doesn't pan out, we are going to assume that the pathname is of the form /the-homedirectory/possible-sub-directory/
    //    so the home directory will be /the-homedirectory/

    const slashChar = "/";
    const backslashChar = "\\"; // Really means one backslash
    let currentPath = window.location.pathname;

    for (const hailMaryRoot of config.hailMaryRoots) {
        let homeIndex = currentPath.indexOf(`${hailMaryRoot}${slashChar}`);
    if (homeIndex < 0) {
        // Maybe some Windows weirdness is going on
            let windowsHailMaryRoot = hailMaryRoot.replaceAll(slashChar, backslashChar);
            homeIndex = currentPath.indexOf(`${windowsHailMaryRoot}${backslashChar}`);
    }
    if (homeIndex >= 0) {
            homeDirectoryCache = currentPath.slice(0, homeIndex) + `${hailMaryRoot}${slashChar}`;
            sessionStorage.setItem(config.homeDirectoryKey, homeDirectoryCache);
        return;
        }
    }

    // If we got here, we didn't find config.hailMaryRoot, so we basically just take the first component of our
    // current pathname as the home directory

    let slashIndex = nthIndex(currentPath, slashChar, 2);
    if (slashIndex < 0) { // Couldn't find a "/"
        // Maybe some Windows weirdness is going on
        let slashIndex = nthIndex(currentPath, backslashChar, 2);
        if (slashIndex < 0) {
            return;
        }
    }
    homeDirectoryCache = currentPath.slice(0, slashIndex) + slashChar;
    sessionStorage.setItem(config.homeDirectoryKey, homeDirectoryCache);
}

function homeDirectory() {
    if (homeDirectoryCache === null) {
        homeDirectoryCache = sessionStorage.getItem(config.homeDirectoryKey);
    }
    if (homeDirectoryCache === null) {
        homeDirectoryHailMaryInit();
        console.log(`initalizeHomeDirectory hail mary, home directory = ${homeDirectoryCache}`);
    }
    return homeDirectoryCache;
}

export {initializeHomeDirectory, homeDirectory};