import {ItemPlacement} from "./item_placement_class.js";
import * as inventory from "./inventory.js";
import {inventory_item_attributes} from "./inventory_config.js";
import * as gamestate from "./gamestate.js";

class InventoryArrangementUI {

    // Parameters from constructor
    originalBackgroundSize ;
    backgroundDivID ;
    inventoryLocationsKey;
    scalingKey;
    itemDivClass;

    constructor(config) {
        this.originalBackgroundSize = config.originalBackgroundSize;
        this.backgroundDivID        = config.backgroundDivID;
        this.inventoryLocationsKey  = config.inventoryLocationsKey;
        this.scalingKey             = config.scalingFactorKey;
        this.itemDivClass           = config.itemDivClass;
    }

    // Working variables
    itemPlacement;
    itemTypeMap = {}; // associates Div ID with item type

    assignNewLocationToItem(location, itemID) {
        const {itemName, itemNumber} = this.itemTypeMap[itemID];
        let inv = inventory.getInventory();

        inv[itemName][this.inventoryLocationsKey][itemNumber] = location;
        gamestate.updateGameState();
    }

    getDisplayedItems() {
        // Gets displayed items from inventory
        let inv = inventory.getInventory();
        let displayedItems = [];
        for (const [itemName, itemProperties] of Object.entries(inv)) {
            if (this.inventoryLocationsKey in itemProperties) {
                let itemNumber = 0;
                for (const location of itemProperties[this.inventoryLocationsKey]) {
                    let displayItemID = `inventoryItem-${itemName}-${itemNumber}`;
                    let image = `../images/${inventory_item_attributes[itemName].image}`;

                    // Scaling in config file is understood as a percent of the width of the "originalBackgroundSize"
                    // so we convert to pixels to avoid any ambiguity in the css
                    let scaling = parseFloat(inventory_item_attributes[itemName][this.scalingKey]) / 100.0;

                    let draggable = true;
                    displayedItems.push({displayItemID, image, location, scaling, draggable});
                    this.itemTypeMap[displayItemID] = {itemName, itemNumber};
                    itemNumber++;
                }
            }
        }
        return displayedItems;
    }

    onDocumentReady() {
        let displayItems = this.getDisplayedItems();
        let itemPlacementConfig = {
             originalBackgroundSize: this.originalBackgroundSize, // size of background image (before scaling)
             backgroundDivID:        this.backgroundDivID,        // ID of div containing background image
             itemDivClass:           this.itemDivClass,           // Class assigned to each displayed item (for styling)
             displayItems,
             moveHandler:            (newLocation, itemID) => this.assignNewLocationToItem(newLocation, itemID)
        };
        this.itemPlacement = new ItemPlacement(itemPlacementConfig);
        this.itemPlacement.onDocumentReady();
    }
}

export {InventoryArrangementUI};