// Example inventory config file
// All possible inventory items must appear as keys in the inventory_item_attributes object.
// Each key points to a set of attributes; the actual attributes specified are up to the developer.
// Attributes are constant properties of each inventory items. The properties that can vary (like count)
// are stored in the inventory object, defined in the inventory.js module.

const initial_item_property_values = {
    count: 1,
    displayed: 0
};

const inventory_item_attributes = {
    bauble: {
        display_name: "bauble",
        category: "jewelry",
        price: 10,
        max_owned: 100,
        prefix: "a "
    },

    // Food
    buffalo_burger: {
        display_name: "buffalo burger",
        category: "food",
        image: "images/burger_no_cheese.png",
        full_width: "small",
        price: 10,
        max_owned: 10,
        prefix: "a "
    },
    buffalo_cheese_burger: {
        display_name: "buffalo cheese burger",
        category: "food",
        image: "images/burger_cheese.png",
        full_width: "small",
        price: 11,
        max_owned: 10,
        prefix: "a "
    },
    fry_bread_taco: {
        display_name: "fry bread taco",
        category: "food",
        image: "images/frybread_taco.png",
        full_width: "medium",
        price: 9,
        max_owned: 10,
        prefix: "a "
    },
    hotdog: {
        display_name: "hotdog",
        category: "food",
        image: "images/hot_dog.png",
        full_width: "small",
        price: 3,
        max_owned: 12,
        prefix: "a "
    },
    chips: {
        display_name: "chips",
        category: "food",
        image: "images/chips.png",
        full_width: "smaller",
        price: 1,
        max_owned: 20,
        prefix: "some "
    },
    waskuya_corn_soup: {
        display_name: "waskuya corn soup",
        category: "food",
        image: "images/soup_frybread.png",
        full_width: "medium",
        price: 9,
        max_owned: 10,
        prefix: "some "
    },
    turkey_leg: {
        display_name: "turkey leg",
        category: "food" ,
        image: "images/turkey_leg.png",
        full_width: "medium",
        price: 11,
        max_owned: 10,
        prefix: "a "
    },
    wojapi_chokecherry_pudding: {
        display_name: "wojapi chokecherry pudding",
        category: "food",
        image: "images/chokecherry_plain.png",
        full_width: "small",
        price: 3,
        max_owned: 10,
        prefix: "some "
    },
    wojapi_with_ice_cream: {
        display_name: "wojapi with ice cream",
        category: "food",
        image: "images/chokecherry_ala_mode.png",
        full_width: "small",
        price: 5,
        max_owned: 10,
        prefix: "some "
    },
    vanilla_ice_cream: {
        display_name: "vanilla ice cream",
        price: 3,
        max_owned: 10,
        prefix: "a "
    },
    strawberry_ice_cream: {
        display_name: "strawberry ice cream",
        category: "food",
        image: "images/strawberry_ice_cream.png",
        full_width: "small",
        price: 3,
        max_owned: 10,
        prefix: "a "
    },
    regular_lemondade: {
        display_name: "regular lemondade",
        category: "food",
        image: "images/regular_lemonade.png",
        full_width: "small",
        price: 5,
        max_owned: 10,
        prefix: "a "
    },
    elderberry_lemonade: {
        display_name: "elderberry lemonade",
        category: "food",
        image: "images/elderberry_lemonade.png",
        full_width: "small",
        price: 5,
        max_owned: 10,
        prefix: "an "
    },

    // Jewelry
    flower_necklace: {
        display_name: "flower necklace",
        category: "jewelry",
        image: "images/necklace_flower.png",
        full_width: "medium",
        price: 60,
        max_owned: 1,
        prefix: "a "
    },
    blue_star_necklace: {
        display_name: "blue star necklace",
        category: "jewelry",
        image: "images/star_necklace_blue.png",
        full_width: "medium",
        price: 60,
        max_owned: 1,
        prefix: "a "
    },
    purple_star_necklace: {
        display_name: "purple star necklace",
        category: "jewelry",
        image: "images/star_necklace_purple.png",
        full_width: "medium",
        price: 60,
        max_owned: 1,
        prefix: "a "
    },
    yellow_star_necklace: {
        display_name: "yellow star necklace",
        category: "jewelry",
        image: "images/star_yellow.png",
        full_width: "medium",
        price: 60,
        max_owned: 1,
        prefix: "a "
    },
    green_and_yellow_earrings: {
        display_name: "green and yellow earrings",
        category: "jewelry",
        image: "images/earrings1_wear.png",
        full_width: "small",
        price: 20,
        max_owned: 1,
        prefix: "some ",
        parts: 2
    },
    green_and_red_earrings: {
        display_name: "green and red earrings",
        category: "jewelry",
        image: "images/earrings_2.png",
        full_width: "small",
        price: 20,
        max_owned: 1,
        prefix: "some ",
        parts: 2
    },
    orange_and_blue_earrings: {
        display_name: "orange and blue earrings",
        category: "jewelry",
        image: "images/earrings_3.png",
        full_width: "small",
        price: 20,
        max_owned: 1,
        prefix: "some ",
        parts: 2
    },
    diamond_earrings: {
        display_name: "diamond earrings",
        category: "jewelry",
        image: "images/earrings_4.png",
        full_width: "small",
        price: 20,
        max_owned: 1,
        prefix: "some ",
        parts: 2
    },
    flower_earrings: {
        display_name: "flower earrings",
        category: "jewelry",
        image: "images/earrings_5.png",
        full_width: "small",
        price: 20,
        max_owned: 1,
        prefix: "some ",
        parts: 2
    },
    red_and_blue_hairclip: {
        display_name: "red and blue hairclip",
        category: "jewelry",
        image: "images/hair_clip.png",
        full_width: "small",
        price: 40,
        max_owned: 1,
        prefix: "a "
    },
    purple_hairclip: {
        display_name: "purple hairclip",
        category: "jewelry",
        image: "images/hair_clip2.png",
        full_width: "small",
        price: 40,
        max_owned: 1,
        prefix: "a "
    },
    yellow_bracelet: {
        display_name: "yellow bracelet",
        category: "jewelry",
        image: "images/bracelet1.png",
        full_width: "small",
        price: 40,
        max_owned: 1,
        prefix: "a "
    },
    red_white_and_blue_bracelet: {
        display_name: "red white and blue bracelet",
        category: "jewelry",
        image: "images/bracelet2.png",
        full_width: "small",
        price: 40,
        max_owned: 1,
        prefix: "a "
    },
    black_and_red_bracelet: {
        display_name: "black and red bracelet",
        category: "jewelry",
        image: "images/bracelet3.png",
        full_width: "small",
        price: 40,
        max_owned: 1,
        prefix: "a "
    },
    // Gifts 
    coup_stick: {
        display_name: "coup stick",
        category: "tchotchke",
        image: "images/coup_stick.png",
        full_width: "medium",
        price: 60,
        max_owned: 1,
        prefix: "a "
    },
    dream_catcher: {
        display_name: "dream catcher",
        category: "tchotchke",
        image: "images/dream_catcher.png",
        full_width: "medium",
        price: 60,
        max_owned: 1,
        prefix: "a "
    },
    wolf_painting: {
        display_name: "small wolf painting",
        category: "tchotchke",
        image: "images/art_wolf.png",
        full_width: "small",
        price: 60,
        max_owned: 1,
        prefix: "a "
    },
    parfleche: {
        display_name: "parfleche",
        category: "tchotchke",
        image: "images/parfleche.png",
        full_width: "small",
        price: 60,
        max_owned: 1,
        prefix: "a "
    },
    feather_painting: {
        display_name: "framed feather painting",
        category: "tchotchke",
        image: "images/framed_feather_art.png",
        full_width: "small",
        price: 200,
        max_owned: 1,
        prefix: "a "
    },
    tomahawk: {
        display_name: "tomahawk",
        category: "tchotchke",
        image: "images/tomahawk.png",
        full_width: "medium",
        price: 200,
        max_owned: 1,
        prefix: "a "
    },
    painting_of_woman: {
        display_name: "framed painting",
        category: "tchotchke",
        image: "images/art_woman.png",
        full_width: "small",
        price: 200,
        max_owned: 1,
        prefix: "a "
    },
    arrows_and_quiver: {
        display_name: "quiver of arrows",
        category: "tchotchke",
        image: "images/arrows_and_quiver.png",
        full_width: "small",
        price: 200,
        max_owned: 1,
        prefix: "a "
    },
    pottery: {
        display_name: "fancy pot",
        category: "tchotchke",
        image: "images/pottery.png",
        full_width: "small",
        price: 60,
        max_owned: 2,
        prefix: "a "
    },

};

export {inventory_item_attributes, initial_item_property_values};