// inventory_ui

import * as inventory from "./inventory.js";
import * as test_inventory from "./test_inventory.js";
import {filterEntriesByAttribute, getInventory} from "./inventory.js";
import * as config from "./inventory_ui_config.js";

let cancelStopDragHandler = false;
let countDisplay = {};

function initializeInventoryMenu(category, test = false) {

    let owned = inventory.getInventory();
    if (test) {
        owned = test_inventory.test_inventory();
    }
    let owned_entries = inventory.filterEntriesByAttribute(config.category_attribute, category, owned);

// Makes the inventory_div  div a dialog box, that will open at bottom right of div in which it  placed ;
    let $inventory_div = $(`#${config.inventory_div_id}`);
    $inventory_div.dialog({
        autoOpen: false,
        draggable: false,
        show: "fade",
        position: {my: "right bottom", at: "right bottom", of: "div"}
    });

// Append to inventory div;
    for (const [itemName, detail] of owned_entries) {
        const attr = inventory.attributesForItem(itemName);
        const img_src = attr.image;
        let $inventory_item_div = $("<div></div>").addClass("inventory_item");
        const this_img = `<img src="${img_src}" class="inventory" alt="${itemName}" data-keyname="${itemName}">`
        const this_count = `<div class="inventory_count" data-item="${itemName}">${detail.count - detail.displayed}</div>`
        $inventory_item_div.append(this_img);
        $inventory_item_div.append(this_count);
        $inventory_div.append($inventory_item_div);
        countDisplay[itemName] = $inventory_div.find(`[data-item="${itemName}"]`);
    }

// When clicked, the inventory div will show with an image for each item owned ;

    $("#inventory").click(function () {
        $("#inventory_div").dialog("open");
    });

// When an item in the inventory is clicked it adds a larger size to the container and makes that image draggable ;
    $(".inventory").click(function () {
        const itemName = $(this).data("keyname");
        const itemDetail = owned[itemName];
        const attr = inventory.attributesForItem(itemName)
        // ??? If the following test fails, right now we just do nothing; probably we should either send a message
        //     or remove the image from the inventory box
        if (itemDetail.displayed < itemDetail.count) {
            const index = itemDetail.displayed;
            itemDetail.displayed++; // ??? Should we remove the image if this reaches count>?
            inventory.updateDB();
            const img_src = attr.image;
            const this_id = itemName + itemDetail.displayed;
            const this_img = `<img src="${img_src}" class="stuff" alt="${itemName}" data-keyname="${itemName}" data-index="${index}" id="${this_id}">`;
            $("#container").append(this_img);

            create_displayed_item_entry(this_id, itemName);

            $(".stuff").draggable({start:start_drag_item, stop: stop_drag_item});
            countDisplay[itemName].html(itemDetail.count - itemDetail.displayed);
        }
    });
}

function initializePlacesMenu() {
    let place_entries = Object.entries(config.places);

    for(const [place, p] in place_entries) {
        $(p.divID).click(() => inventory.safe_go_to(p.url));
    }

    $("#mystuff").on('click', function () {
        $('#myplaces').show();
    });
    $("#close").on('click', function () {
        $('#myplaces').hide();
    });

}

function create_displayed_item_entry(element_id, itemName) {
    let detail = getInventory()[itemName];
    const attr =  inventory.attributesForItem(itemName);

    if (!("displayed_items" in detail)) {
        detail.displayed_items = [];
    }

    detail.displayed_items.push({offset: {...$("#" + element_id).offset()}});
    inventory.updateDB();
}

function initializeDisplayedItems(category) {
    let theInventory = getInventory();
    let owned_entries = inventory.filterEntriesByAttribute("category", category);
    for (const [itemName, detail] of owned_entries) {
        if (detail.displayed > 0) {
            for (let index = 0; index < detail.displayed; index++) {
                const attr = inventory.propertiesForItem(itemName);
                const img_src = attr.image;
                let this_id = itemName + index;
                let this_img = `<img src="${img_src}" class="stuff" alt="${itemName}" data-keyname="${itemName}" data-index="${index}" id="${this_id}">`;
                $("#container").append(this_img);
                $("#" + this_id).css("position", "absolute").offset(detail.displayed_items[index].offset);
            }
        }
    }

    $(".stuff").draggable({stop: stop_drag_item});
    $("#inventory").droppable({drop: put_back_item});
    $("#inventory_div").droppable({drop: put_back_item});
}

function removeDisplayedItemsAtQuota(category, action) {
    let entries = inventory.filterEntriesByAttribute(category_attribute, category);
    for (const [itemname, detail] of entries) {
        let attr = inventory.attributesForItem(itemname);
        if (inventory.itemCount(itemname) >= attr.max_owned) {
            action(itemname);
        }
    }
}


function start_drag_item(event, ui) {
    cancelStopDragHandler = false;
    let thePosition = ui.helper.position();
    let theOffset = ui.helper.offset();
    console.log(`position = ${thePosition.top}, ${thePosition.left}, offset  ${theOffset.top}, ${theOffset.left}`);
    ui.helper.css("position", "absolute");
    ui.helper.css("top", thePosition.top);
    ui.helper.css("left", thePosition.left);
}

function stop_drag_item(event, ui) {
    if (cancelStopDragHandler) {
        // If the droppable handler "put_back_item" already ran, then don't run this handler
        cancelStopDragHandler = false;
        return;
    }
    const index = ui.helper.data("index");
    const item = ui.helper.data("keyname");
    let detail = inventory.getInventory()[item];
    if (!("displayed_items" in detail)) {
        return;
    }
    let displayed_items = detail.displayed_items;
    if (displayed_items === undefined) {
        console.log ("In stop_drag_item, displayed_items was undefined");
        return;
    }

    if (index >= detail.displayed) {
        console.log(`index greater than size of displayed_items array, returning`);
        return;
    }
    displayed_items[index].offset = {...$(ui.helper).offset()};

    inventory.updateDB();
}

function put_back_item(event, ui) {
    cancelStopDragHandler = true;
    const itemName = ui.draggable.data("keyname");
    const detail = getInventory()[itemName];
    const index = ui.draggable.data("index");
    let displayed_items = detail.displayed_items;

    ui.draggable.remove();
    detail.displayed--;
    let lastItem = displayed_items.pop();
    let isLastItem = (Number(index) === Number(detail.displayed));

    // If we put back the last item then we are done; if not then we will copy the last item into the index of
    // item removed

    if (!isLastItem) {
        // Re-label display item to replace deleted item
        let theID = itemName + index;
        let theLastID = "#" + itemName + detail.displayed;
        $(theLastID).data("index", index);
        $(theLastID).prop("id", theID);

        // Modify array in inventory
        displayed_items[index] = {offset: {...lastItem.offset}};
    }
    countDisplay[itemName].html(detail.count - detail.displayed);

    inventory.updateDB();
}

export {initializeInventoryMenu, initializePlacesMenu, initializeDisplayedItems, removeDisplayedItemsAtQuota};