// milestone.js
// module

// There are two ways to declare a milestone on a page:
//
// 1) Define a value for the variable "MilestoneForThisPage", then include this module
// 2) Include this module, importing declareMilestoneForThisPage, and then call declareMilestoneForThisPage
//    with the name of the milestone for this page.
//
// The first method does not allow a callback to be specified to assure that the page is not left before the
// milestone is written to the database.
//
// In the second method, declareMilestoneForThisPage returns a promise so the page calling it can make
// sure the database transaction is completed by attaching any navigation to a then clause


import {milestones, defaultMilestone} from "./milestone_config.js";
import * as gamestate from "./gamestate.js";
import $ from "./jquery.module.js";


$(document).ready(function() {
    if ((typeof MilestoneForThisPage) != "undefined") {
        updateMilestone(MilestoneForThisPage);
    }
});

function updateMilestone(milestone, callback) {
    if (milestone in milestones) {
        let g = gamestate.getGameState();
        g.milestone = milestone;
        return gamestate.updateGameState(callback);
    }
    else {
        alert("Milestone " + milestone + " does not exist!");
        return null;
    }
}

function goToCurrentMilestone() {
    let g = gamestate.getGameState();
    let milestone = g.milestone;
    if (milestone) {
        let theURL = milestones[milestone];
        if (theURL) {
            window.location.href = theURL;
        }
    }
    else {
        g.milestone = defaultMilestone;
        gamestate.updateGameState(() => window.location.href = milestones[defaultMilestone]);
    }
}

function declareMilestoneForThisPage(milestoneForThisPage) {
    return updateMilestone(milestoneForThisPage);
}

function currentMilestone() {
    let g = gamestate.getGameState();
    let milestone = g.milestone;
    return milestone ? milestone : defaultMilestone;
}

export {goToCurrentMilestone, declareMilestoneForThisPage, currentMilestone, milestones};
export {safe_go_to} from "./gamestate.js";