import * as audio from "./audio.js";
import SetConfig from "./set_config_class.js";
import $ from "./jquery.module.js";


/*
  Will handle getting the values from the config file and making these available in this file with the config.____ format
 */
let setConfigHandler = new SetConfig();
let config ;
function setConfig(config) {
    setConfigHandler.setConfig(config);
}
$(document).ready(function () {
    setConfigHandler.then((configuration)=> {
        let config = {...configuration}; // Shallow copy because configuration is const, and we need to
        //Button click to start the game to avoid autoplay
        $("#layer1").css("background-image", "url(" + config.options.background + ")");

        let whacktime = 0;
        let audio1 = config.options.soundtrack;
       let supportsTouch = 'ontouchstart' in window || navigator.msMaxTouchPoints;

        let target = {
            // image file and size of each frame
            g_image: config.options.the_mole,
            image_width: config.options.frame_width,
            image_height: config.options.frame_height,

            // Number of animation frames and starting frame
            numberOfFrames: 2, //only for the jumping animation
            currentFrame: 0,

            // Initial X & Y positions on sprite sheet
            sourceX: 0,
            sourceY: 0,

            // Is it jumping out of the hole?
            forward: true,

            // States
            hiding: 0,
            jumping: 1,
            hit: 2,
            state: 0, // hiding

            // Properties for resetting counter after hit;
            timeToReset: 30,
            resetCounter: 0,

            // Property to store random time
            waitTime: undefined,

            // Method to find random animation time
            findWaitTime: function () {
                this.waitTime = Math.ceil(Math.random() * 30);
            },
            updateAnimation: function () {
                // Use currentFrame to find correct image in sheet
                this.sourceX = this.currentFrame * this.image_width;
                this.sourceY = 0;

                // Find state
                if (this.state !== this.hit) {
                    if (this.waitTime > 0 || this.waitTime === undefined) {
                        this.state = this.hiding;
                    } else {
                        this.state = this.jumping;
                    }
                }

                // Switch creature's actions based on its state
                switch (this.state) {
                    case this.hiding:
                        this.currentFrame = 0;
                        this.waitTime--;
                        break;
                    case this.jumping:
                        if (this.currentFrame === 8) {
                            this.findWaitTime();
                            this.state = this.hiding;
                            this.currentFrame = 0; //reset running animation
                            break;
                        }
                        this.currentFrame = 1;
                        break;
                    case this.hit:
                        this.currentFrame = 9;
                        if (whacktime === 0) {
                            whacktime++;
                        }
                        // update resetCounter
                        this.resetCounter++;
                        // Reset animation when time is up
                        if (this.resetCounter >= this.timeToReset) {
                            whacktime = 0;
                            this.resetCounter = 0;
                            this.state = this.hiding;
                            this.forward = true;
                            this.currentFrame = 0;
                            this.findWaitTime();
                        }
                        break;
                    default:
                        break;
                }
            },
        };

        let targetsHit = 0;

// Create the timer
        let gameTimer = {
            time: 0,
            interval: undefined,

            start: function () {
                let self = this;
                this.interval = setInterval(function () {
                    self.tick();
                }, 1000);
            },
            tick: function () {
                this.time--;
            },
            stop: function () {
                clearInterval(this.interval);
            },
            reset: function () {
                this.time = 0;
            }
        };

        let stage = document.getElementById('stage');
        let output = document.getElementById('output');
        let color =  config.options.scoreColor ;


// Rows and columns for building a map of holes
        let row_num = config.options.row_num;
        let col_num = config.options.col_num;
        let space = config.options.space;

// Arrays for targets and their drawing
        let targetObjects = [];
        let targetCanvases = [];
        let targetDrawingSurfaces = [];
        let width = target.image_width;
        let height = target.image_height;
        let canvas = document.querySelector("canvas");
        let drawingSurface = canvas.getContext("2d");
        canvas.addEventListener("mousedown", mousedownHandler, false);


// Load animation sheet
        let image = new Image();
        image.addEventListener("load", loadHandler, false);
        image.src = target.g_image;

// Set up on load
        function loadHandler() {
            // add the targets to the page
            buildMap();

            // start timer
            gameTimer.time = 30;
        }

        function updateAnimation() {
            // set timer to call update
            if (gameTimer.time > 0) {
                setTimeout(updateAnimation, 150);
            }

            //update target animation
            for (let i = 0; i < targetObjects.length; i++) {
                targetObjects[i].updateAnimation();
            }
            if (gameTimer.time === 0) {
                endGame();
            }

            // draw it
            render();
        }

        function endGame() {
            gameTimer.stop();
            audio.setAudioFile("") ;
            for (let i = 0; i < targetCanvases.length; i++) {
                let canvas = targetCanvases[i];
                canvas.removeEventListener("mousedown", mousedownHandler, false);
            }
            $("#stage").hide(); //hides game, shows endgame div
            $("#output").hide(); //hides game, shows endgame div
            $("#endgame").html("<div class='bubble'><h1>Your score: " + targetsHit + "</h1><br/><img src='../../game_theme/theme/icons/continue.png' alt='continue' id='arrow_right'></div>").show();
            $("#congrats").show();
            $("#winimg").show();
            $("#arrow_right").on("click",() => {
                window.location.href = config.options.completionURL ;
            });
        }

        function mousedownHandler(event) {
            // Find which canvas has been clicked
            let theCanvasThatWasClicked = event.target;
            for (let i = 0; i < targetCanvases.length; i++) {
                if (targetCanvases[i] === theCanvasThatWasClicked) {
                    let target = targetObjects[i];
                    if (target.state === target.jumping) {
                        target.state = target.hit;
                        targetsHit++;
                    }
                }
            }
        }

        function render() {
            for (let i = 0; i < targetObjects.length; i++) {
                let target = targetObjects[i];
                let drawingSurface = targetDrawingSurfaces[i];

                drawingSurface.clearRect(0, 0, width * 2, width * 2);
                drawingSurface.drawImage(image, target.sourceX, target.sourceY, width, height, 0, 0, width * 2, height);
            }
            output.innerHTML = "whacked: " + targetsHit + " , " + "Time Left: " + gameTimer.time;
            output.style.setProperty('color', color) ;
            console.log(color)
        }

        function buildMap() {
            let offsetV = 160;
            let offsetH = 20;

            for (let row = 0; row < row_num; row++) {
                for (let column = 0; column < col_num; column++) {
                    // Create new object and add to array
                    let newtargetObject = Object.create(target);
                    newtargetObject.findWaitTime();
                    targetObjects.push(newtargetObject);

                    // Create canvas for each target and add to stage
                    // Add mousedown listen for each canvas
                    let canvas = document.createElement('canvas');
                    canvas.setAttribute("width", width * 2);
                    canvas.setAttribute("height", width * 2);
                    stage.appendChild(canvas);
                    canvas.style.top = offsetV + row * (height + space) + "px";
                    canvas.style.left = offsetH + column * (width + space) + "px";
                    if (supportsTouch) {
                        canvas.addEventListener("touchstart", mousedownHandler, false);
                    } else {
                        canvas.addEventListener("mousedown", mousedownHandler, false);
                    }
                    targetCanvases.push(canvas);

                    // Create drawing surface
                    let drawingSurface = canvas.getContext("2d");
                    targetDrawingSurfaces.push(drawingSurface);
                }
            }
        }


        let intro = 0;
        let instructions = config.options.instructions;

        //let time = config.options.seconds;
        let start = document.getElementById("start");

        $('#intro_text').text(instructions[0]);

        start.addEventListener("click", () => {
            intro++;
            if (intro === instructions.length) {
                $("#instructions").hide();
                stage.style.setProperty('display', 'flex') ;
                gameTimer.start();

                // start animation
                updateAnimation();
                audio.setAudioFile(audio1) ;
                audio.playAudioFile(audio1) ;

            } else {
                $('#intro_text').text(instructions[intro]);
            }
        });
    });
});
export {setConfig};