import * as username from "./username.js";
import * as db from "./database_access.js";
import * as gamestate from "./gamestate.js";

// writeUserActivityLogRecord
//    writes directly to database
//
// writeUserActivityLogRecordProvisional
//    uses the database module provisional feature to save data that
//    may be incomplete. Each time this is called the data is updated.
//    When you leave the page with safe_go_to, the last update is written
//    to the database.

function writeUserActivityLogRecord(fieldValues) {
    // fieldValues is an object containing the following fields:
    // fieldsValues
    //  = {
    //       activity_name:       (text),
    //       questions_attempted: (integer),
    //       questions_correct:   (integer),
    //    }

    fieldValues.userid  = username.getUserName();
    fieldValues.session = gamestate.getSession();
    fieldValues.action  = "write-user-activity-record";
    return db.perform_db_action_promise(fieldValues);
}

function writeUserActivityLogRecordProvisional(fieldValues) {
    // fieldValues is an object containing the following fields:
    // fieldsValues
    //  = {
    //       activity_name:       (text),
    //       questions_attempted: (integer),
    //       questions_correct:   (integer),
    //    }

    fieldValues.userid  = username.getUserName();
    fieldValues.session = gamestate.getSession();
    fieldValues.action  = "write-user-activity-record";
    return db.setProvisionalData(fieldValues);
}

function submitUserActivityLogForm(formID) {
    // For some reason we put all of these values into hidden form elements instead of JavaScript variables...

    return writeUserActivityLogRecord({
        activity_name:       $(formID + " input[name=activity_name]").val(),
        questions_attempted: $(formID + " input[name=questions_attempted]").val(),
        questions_correct:   $(formID + " input[name=questions_correct]").val(),
    });

}

export {writeUserActivityLogRecord, submitUserActivityLogForm, writeUserActivityLogRecordProvisional};
export {safe_go_to, safe_reload_page} from "./gamestate.js";
